/*
fuzzy_tsn.hh

This C++ header file defines the NML Messages used for command and status by FUZZY_TS_MODULE
Template Version 1.1

MODIFICATIONS:
Fri Dec 15 15:49:32 EST 2000	Modified by rcsdesign.
Thu Dec 14 20:30:48 EST 2000	Created by rcsdesign.

*/

// Prevent Multiple Inclusion
#ifndef FUZZY_TSN_HH
#define FUZZY_TSN_HH

// Include Files
#include "rcs.hh" 	// Common RCS definitions

// Predefined type files
// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

#define NUM_OF_INPUTS 2
#define NUM_OF_TS_INPUTS 2

// Define the integer type ids.
//  RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

#define FUZZY_TS_STATUS_TYPE 1000
#define FUZZY_TS_HALT_TYPE 1001
#define FUZZY_TS_INIT_TYPE 1002
#define FUZZY_TS_RUN_TYPE 1003
#define FUZZY_TS_SET_ADAPTATION_GAIN_TYPE 1004
#define FUZZY_TS_SET_INPUT_GAINS_TYPE 1005
#define FUZZY_TS_SET_IN_MF_TYPE 1006
#define FUZZY_TS_SET_IN_UD_TYPE 1007
#define FUZZY_TS_SET_PREMISE_TYPE 1008
#define FUZZY_TS_SET_TS_COEFF_TYPE 1009
//  RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


// Define the NML Message Classes

// Status Class
class FUZZY_TS_STATUS : public RCS_STAT_MSG
{
public:

	// Normal Constructor
	FUZZY_TS_STATUS();

	// Constructor used by derived classes
	FUZZY_TS_STATUS(NMLTYPE t, size_t s) :  RCS_STAT_MSG(t,s) {};

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	double sys_inputs[NUM_OF_INPUTS];
	double ts_inputs[NUM_OF_TS_INPUTS];
	double fuzzy_ts_output;

	int premise_type;
	double adapatation_gain;

	// Each cycle the status of different input ud will be passed 
	int num_of_current_in_ud;
	double gain_for_in_ud;
	int num_of_current_in_mf;
	int type_of_current_in_mf;
	double center_of_current_in_mf;
	double spread_of_current_in_mf;

	// Each cycle the output parameters of different rule will be passed 
	int num_of_current_rule;
	double ts_coeff[NUM_OF_TS_INPUTS+1];

};

// Command Classes

class FUZZY_TS_RUN : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_RUN();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.

};

class FUZZY_TS_HALT : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_HALT();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.

};

class FUZZY_TS_INIT : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_INIT();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.

};

class FUZZY_TS_SET_ADAPTATION_GAIN : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_SET_ADAPTATION_GAIN();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	// the adaptation gain eta
	double eta;
};

class FUZZY_TS_SET_INPUT_GAINS : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_SET_INPUT_GAINS();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	double input_gains[NUM_OF_INPUTS];
};

class FUZZY_TS_SET_IN_MF : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_SET_IN_MF();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	int num_of_current_in_ud;
	int num_of_current_in_mf;
	int mf_type;
	double mf_center;
	double mf_spread;

};

class FUZZY_TS_SET_IN_UD : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_SET_IN_UD();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	int num_of_current_in_ud;
	double gain;
	double first_center;
	double last_center;
	int mf_type;
	double spread;

};

class FUZZY_TS_SET_PREMISE : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_SET_PREMISE();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	int premise_type;

};

class FUZZY_TS_SET_TS_COEFF : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_TS_SET_TS_COEFF();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	int num_of_current_rule;
	double ts_coeff[NUM_OF_TS_INPUTS+1];

};

// Declare NML format function
extern int fuzzy_tsFormat(NMLTYPE, void *, CMS *);

#endif 	// FUZZY_TSN_HH
